/************************************************************************/
/*                                                                      */
/*  Licensed Materials - Property of IBM                                */
/*  WA84 SupportPac                                                     */
/*  (c) Copyright IBM Corp. 2001                                        */
/*  US Government Users Restricted Rights - Use, duplication or         */
/*  disclosure restricted by GSA ADP Schedule Contract                  */
/*  with IBM Corp.                                                      */
/*                                                                      */
/************************************************************************/
package com.ibm.workflow.servlet.clientx.wext;

import javax.servlet.http.*;
import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ProcessInstancePackage.ExecutionState;
import com.ibm.workflow.servlet.client.*;

/**
* This class provides a function to reuse the data of "finished" process input container
* for the instantiation of new process instance.
*/
public class ReuseProcessDataHandler extends WExtCommandHandler {
    private static final WExtCommand cmd = new WExtCommand("reuseAndStartInstance", "reuse.gif", "Command.ReuseAndStartInstance");

    /**
    * Returns information about the handler.
    * @return   a String containing handler information
    */
    public String getHandlerInfo() {
        return "MQWF WebClient Extentions V1.0 - ReuseProcessDataHandler";
    }

    /**
    * Initializes the handler.
    * @param config     the servlet's configuration and initialization parameters
    */
    public void init(Config cfg) {
        super.init(cfg);
        getContext().log("init ReuseProcessHandler", false);
    }

    /**
    * Returns a String that contains HTML code to trigger the reuse process data command.
    * @param list    denotes the list for which the trigger tag is queried.
    * @param context the RequestContext which was passed to the Viewer
    *                calling this method.
    * @return A string containing HTML tags for the icons to be displayed on list.
    */
    public String getTriggerTagFor(int list, RequestContext context) {
        if (list != SessionContext.INSTANCELIST)
            return "";

        try {
            ProcessInstance instance = context.getInstance();
            if (instance == null) {
                context.log(" ProcessInstance is null", false);
                return "";
            }

            if (instance.state().value() != ExecutionState.FINISHED.value())
                return "";

            String oid = instance.persistentOid();
            String name = instance.name();
            return cmd.getTriggerTag(context, oid, name);
        }
        catch (FmcException ex) {
            return "";
        }
    }

    /**
    * This custom command first retrieves the input container data of the prosess
    * instance whose OID matches the 'id' parameter in the HTTP-request.
    * Then the command creates a ReuseData object as a request bean to keep old data for reuse.
    * The resulting HTML data is created by createAndStartInstance rendering.
    * @param request        the request the client has made of the servlet
    * @param response       the response the servlet sends to the client
    * @return               the result page to be sent to the client
    */
    public ResponsePage reuseAndStartInstance(HttpServletRequest request,
                                            HttpServletResponse response)
                                        throws ClientException {
        getContext().log("createAndStartInstance called.", false);
        try {
            SessionContext sc = getSessionContext(request);
            BuiltinHandler bh = sc.getBuiltinHandler();
            RequestContext context = new RequestContext(request);
            String id = request.getParameter("id");
            ProcessInstance pinstance = sc.getInstance(sc.getInstanceListOid(), id);
            if (pinstance == null) {
                getContext().log("  Process " + id + " not found.", false);
                throw new ClientException(request, context.get("Error.InstanceNotFound ", id));
            }

            String name = pinstance.processTemplateName();
            ProcessTemplate ptemplate = findTemplate(context, name);
            if (ptemplate == null) {
                getContext().log("  Template " + id + " not found.", false);
                throw new ClientException(request, context.get("Error.TemplateNotFound", id));
            }

            context.setTemplate(ptemplate);

            ReadWriteContainer container = pinstance.inContainer();
            ReuseData data = new ReuseData(container, context);
            request.setAttribute(getDataBeanId(), data);

            ResponsePage page = bh.render("createAndStartInstance", context);
            getContext().log("reuseAndStartInstance done.", false);
            return page;
        }
        catch (FmcException ex) {
            throw new ClientException(request, ex);
        }
    }


    /**
    * Find the template bye the  name
    * @param context    the RequestContext object
    * @param name       a String object of target process template name
    * @return           the ProcessTemplate object
    */
    ProcessTemplate findTemplate(RequestContext context, String name) throws FmcException {
        try {
            ExecutionService service = context.getExecutionService();
            ProcessTemplate[] allTemplates = service.queryProcessTemplates(null, null, null);
            if (allTemplates != null) {
                for (int i = 0; i < allTemplates.length; i++) {
                    ProcessTemplate ptemplate = allTemplates[i];
                    if (ptemplate.name().equals(name)) {
                        ptemplate.refresh();
                        return ptemplate;
                    }
                }
            }
        }
        catch (FmcException ex) {
            if (ex.rc == FmcException.FMC_ERROR_DOES_NOT_EXIST) {
                getContext().log("  Template " + name + " no longer exists, forcing refresh", false);
                return findTemplate(context, name);
            }
            throw ex;
        }
        return null;
    }

}
