/************************************************************************/
/*                                                                      */
/*  Licensed Materials - Property of IBM                                */
/*  WA84 SupportPac                                                     */
/*  (c) Copyright IBM Corp. 2001                                        */
/*  US Government Users Restricted Rights - Use, duplication or         */
/*  disclosure restricted by GSA ADP Schedule Contract                  */
/*  with IBM Corp.                                                      */
/*                                                                      */
/************************************************************************/
package com.ibm.workflow.servlet.clientx.wext;

import java.util.Enumeration;
import java.lang.reflect.Method;
import javax.servlet.http.*;
import com.ibm.workflow.api.*;
import com.ibm.workflow.servlet.client.*;

/**
* This is a base class of WebClinet Ententions handlers.
*/
public abstract class WExtCommandHandler extends GenericCommandHandler {
    private String dataBeanId;
    private String pageBeanId;
    private String pageId;

    /**
    * Initializes the handler, if configured.
    * This feature has to be enabled in WebClient.properties
    * by setting DataBeanID, PageBeanID and PageID.
    * @param config     the servlet's configuration and initialization parameters
    */
    public void init(Config cfg) {
        super.init(cfg);
        dataBeanId = cfg.getParameter("WebClientExtentions", "DataBeanID", "reuseData");
        pageBeanId = cfg.getParameter("WebClientExtentions", "PageBeanID", "pageController");
        pageId = cfg.getParameter("WebClientExtentions", "PageID", "x-pid");
    }

    protected String getDataBeanId() {
        return dataBeanId;
    }

    protected String getPageBeanId() {
        return pageBeanId;
    }

    protected String getPageId() {
        return pageId;
    }

    protected PageController getPageController(HttpServletRequest request) throws ClientException {
        HttpSession session = request.getSession(false);
        if (session == null) {
            getContext().log("  Cannot get session ", false);
            throw new ClientException(request, "No SessionTracking");
        }

        Object obj = session.getValue(getPageBeanId());
        if (obj instanceof PageController) {
            PageController controller = (PageController)obj;
            controller.setLastCommand(request.getParameter("command"));
            return controller;
        }
        return null;
    }


    protected ResponsePage invokeEventListener(Object listener,
                                            String methodName,
                                            RequestContext context,
                                            PageController controller,
                                            PostedValues values) throws ClientException {
        getContext().log("invokeEventListener:" + methodName, false);
        try {
            Class[] paramClasses;
            Object[] param;
            if (values == null) {
                paramClasses = new Class[] { RequestContext.class, PageController.class };
                param = new Object[] { context, controller };
            } else {
                paramClasses = new Class[] { RequestContext.class, PageController.class, PostedValues.class };
                param = new Object[] { context, controller, values };
            }

            Method method = listener.getClass().getMethod(methodName, paramClasses);
            Object pageobj = method.invoke(listener, param);
            if (pageobj instanceof ResponsePage)
                return (ResponsePage)pageobj;
        }
        catch (NoSuchMethodException ex) {
            getContext().log("  Method not found.", false);
        }
        catch (Exception ex) {
            getContext().log("  listener rise ", ex);
            throw new ClientException(context.getRequest(), ex);
        }
        return null;
    }

    protected void setContainerData(ReadWriteContainer container, ReuseData outData) {
        for (Enumeration enum = outData.getItems(); enum.hasMoreElements();) {
            String name = (String)enum.nextElement();
            String val = outData.getValue(name);

            if (val == null || val.length() == 0)
                continue; // really ?

            try {
                ContainerElement element = container.getElement(name);
                if (element == null)
                    continue;

                getContext().log("    Setting member: " + element.name(), false);
                String type = element.type();
                try {
                    if (type.equals("STRING"))
                        container.setString(name, val);
                    else if (type.equals("LONG"))
                        container.setLong(name, Integer.parseInt(val));
                    else if (type.equals("FLOAT"))
                        container.setDouble(name, Double.parseDouble(val));
                    // else if (type.equals("BINARY"))
                }
                catch (NumberFormatException ex) {
                    getContext().log("      Not a valid number: " + val, false);
                }
            }
            catch (FmcException ex) {
                if (ex.rc != FmcException.FMC_ERROR_MEMBER_NOT_FOUND)
                    getContext().log("Error set " + name + "=" + val, ex);
            }
        }
    }


    protected ResponsePage checkInOrCompleteWorkItem(HttpServletRequest request,
                                HttpServletResponse response,
                                boolean complete) throws ClientException {
        try {
            SessionContext sc = getSessionContext(request);
            BuiltinHandler bh = sc.getBuiltinHandler();
            PageController controller = getPageController(request);
            if (controller == null) {
                RequestContext ctx = bh.checkInWorkItem(request, response);
                return bh.render("checkInWorkItem", ctx);
            }

            RequestContext context = new RequestContext(request);
            String id = request.getParameter("id");
            String retCode = request.getParameter("returnCode");
            WorkItem wi = sc.getWorkItem(sc.getWorkListOid(), id);
            context.setWorkItem(wi);
            context.setContainer(wi.inContainer());

            if (wi == null) {
                getContext().log("  Workitem " + id + " not found.", false);
                throw new ClientException(request, context.get("Error.WorkItemNotFound", id));
            }

            ReadWriteContainer container = wi.outContainer();
            PostedValues values = new PostedValues(request);

            // queryCheckInWorkItem
            ResponsePage page = null;
            Object listener = controller.getEventListener();
            if (listener instanceof CheckInWorkItemEventListener) {
                page = ((CheckInWorkItemEventListener)listener).queryCheckInWorkItem(context, controller, values);
            } else if (listener != null) {
                page = invokeEventListener(listener, "queryCheckInWorkItem", context, controller, values);
            }
            if (page != null) {
                getContext().log(" abort checkInWorkItem", false);
                request.setAttribute(getDataBeanId(), controller.getReuseData());
                return page;
            }

            controller.setPostedValues(values);
            setContainerData(container, controller.getReuseData());

            int rc = 0;
            try {
                if (retCode != null)
                    rc = Integer.parseInt(retCode);
            }
            catch (NumberFormatException numex) {
                getContext().log(wi.name() + ": " + retCode + " is not a valid Long, using returnCode=0", true);
            }

            boolean isManualExit = wi.manualExitMode();
            wi.checkIn(container, rc);
            if (isManualExit && complete) {
                getContext().log(" finish a manual exit mode work item.", false);
                wi.finish();
            }

            // postCheckInWorkItem
            if (listener instanceof CheckInWorkItemEventListener) {
                page = ((CheckInWorkItemEventListener)listener).postCheckInWorkItem(context, controller);
            } else if (listener != null) {
                page = invokeEventListener(listener, "postCheckInWorkItem", context, controller, null);
            }
            if (page == null)
                page = bh.render("checkInWorkItem", context);

            controller.initAll();
            return page;
        }
        catch (FmcException ex) {
            throw new ClientException(request, ex);
        }
    }

    protected ResponsePage cancelOrRetrieveDraftWorkItem(HttpServletRequest request,
                                HttpServletResponse response,
                                boolean retrieve) throws ClientException {
        try {
            SessionContext sc = getSessionContext(request);
            BuiltinHandler bh = sc.getBuiltinHandler();
            PageController controller = getPageController(request);
            if (controller == null
             || controller.getReuseData() == null
             || controller.getReuseData().getOriginalContainer() == null) {
                RequestContext ctx = bh.cancelWorkItem(request, response);
                return bh.render("cancelWorkItem", ctx);
            }

            RequestContext context = new RequestContext(request);
            String id = request.getParameter("id");
            WorkItem wi = sc.getWorkItem(sc.getWorkListOid(), id);
            context.setWorkItem(wi);

            if (wi.manualExitMode() && retrieve) {
                ReadWriteContainer container = controller.getReuseData().getOriginalContainer();
                int rc = container.getLong("_RC");
                wi.checkIn(container, rc);
            } else {
                context = bh.cancelWorkItem(request, response);
            }

            // postCancelWorkItem
            ResponsePage page = null;
            Object listener = controller.getEventListener();
            if (listener instanceof CancelWorkItemEventListener) {
                page = ((CancelWorkItemEventListener)listener).postCancelWorkItem(context, controller);
            } else if (listener != null) {
                page =
                invokeEventListener(listener, "postCancelWorkItem", context, controller, null);
            }

            if (page == null)
                page = bh.render("cancelWorkItem", context);

            controller.initAll();
            return page;
        }
        catch (Exception ex) {
            throw new ClientException(request, ex);
        }
    }
}
